﻿using System;
using System.Collections;
using System.Text;
using System.Drawing;
using System.Data;

namespace RecognizeSignature
{
    class SignCalculateData
    {
        private ArrayList xAxis;
        private ArrayList yAxis;
        private ArrayList normalPressure;
        private ArrayList timeTick;
        private ArrayList histogramP;
        private ArrayList histogramX;
        private ArrayList histogramY;
		private int       numberStrokes;
		private int       numberPixels;
		private int       width;
		private double    length;
		private double    complexity;


		//properties
		public int NumberStrokes
		{
			get{return numberStrokes;}
		}

		public int NumberPixels
		{
			get{ return numberPixels;}
		}

		public int Width
		{
			get{ return width;}
		}

		public double Length
		{
			get{ return length;}
		}

		public double Complexity
		{
			get{ return complexity;}
		}

		// constraction
        public SignCalculateData(ArrayList xAxis, ArrayList yAxis, ArrayList normalPressure, Rectangle boundingBox, int numberStrokes)
        {
            this.xAxis = NormalizeCordinate(xAxis, boundingBox.Left);
            this.yAxis = NormalizeCordinate(yAxis, boundingBox.Top);
            this.normalPressure = normalPressure;
            this.histogramP = Histograma(this.normalPressure, normalPressure.Count-1);
            this.histogramX = Histograma(this.xAxis, (boundingBox.Width-1) );
            this.histogramY = Histograma(this.yAxis, (boundingBox.Height-1) );
			this.numberPixels = xAxis.Count;
			this.numberStrokes = numberStrokes;
			this.width = boundingBox.Width;
			this.length = GetLengthSignature(this.xAxis,this.yAxis);
			this.complexity = length/width;
        }

        public SignCalculateData()
        {
            this.xAxis = new ArrayList();
            this.yAxis = new ArrayList();
            this.normalPressure = new ArrayList();
        }

        ArrayList NormalizeCordinate(ArrayList Cord, int value)
        {
            ArrayList NormalizeData = new ArrayList();
            for (int i = 0; i < Cord.Count; i++)
            {
                int data = (int)Cord[i] - value;
                NormalizeData.Add(data);
            }
            return NormalizeData;
        }
		public Point GetGeometCenter()
		{ 
			int x = 0;
			int y = 0;
            
			for (int i = 0; i < xAxis.Count; i++)
			{
				x += (int)xAxis[i];
				y += (int)yAxis[i];
			}
            
			if( xAxis.Count != 0 )
				x = x / xAxis.Count;
			
			if( yAxis.Count != 0 )
				y = y / yAxis.Count;
			

			return new Point(x, y);
		}

        public Point GetWeightCenter()
        {
            int Px = 0;
            int Py = 0;
            int sumPressure = 0;
            for (int k = 0; k < normalPressure.Count; k++)
            {
                sumPressure += (int)normalPressure[k];
            }

            for (int i = 0; i < xAxis.Count; i++)
            {
                Px += (int)xAxis[i] * (int)normalPressure[i];
                Py += (int)yAxis[i] * (int)normalPressure[i];
            }

			if( sumPressure != 0)
			{
				Px /= sumPressure;
				Py /= sumPressure;
			}

            return new Point(Px, Py);
        }
        /// <summary>
        /// For calsification of Signature
        /// </summary>
        /// <param name="GC"></param>
        /// <param name="WC"></param>
        /// <returns></returns>
        public double GetDistanceCenters()
        {
            Point GC = GetGeometCenter();
            Point WC = GetWeightCenter();
            double distance = 0;
            distance = Math.Pow((GC.X - WC.X), 2) + Math.Pow((GC.Y - WC.Y), 2);
            distance = Math.Pow(distance, 0.5);
            return distance;
        }

        public double GetAngle_GC_WC()
        {
            double angle = 0;

            Point GC = GetGeometCenter();
            Point WC = GetWeightCenter();
            
			double delitel = GC.Y - WC.Y;
			if (delitel == 0)
				return Math.PI/2;
		
            double tangeta = (GC.X - WC.X) / delitel;
            angle   = Math.Atan(tangeta);
			if( angle < 0)
				angle = Math.PI - angle;
            return angle;
        }

        public double GetDispersia(string kind)
        {
            double dispersia = 0;
            switch (kind)
            {
                case "X":
                    dispersia = Dispersia(histogramX);
                    break;
                case "Y":
                    dispersia = Dispersia(histogramY);
                    break;
                case "P":
                    dispersia = Dispersia(histogramP);
                    break;
            }
            
            return dispersia;
        }

        private double Dispersia(ArrayList histogram)
        {
            double sum = 0;
            double centerHistogram = 0;
            double dispersia = 0;

            centerHistogram = GetCenterHistogram(histogram);
            for (int i = 0; i < histogram.Count; i++)
            {
                sum += (int)histogram[i] * Math.Pow((i - centerHistogram), 2);
            }
			if( histogram.Count > 2)
                dispersia = sum / (histogram.Count - 2);
            return dispersia;
        }

        private ArrayList Histograma(ArrayList data,int maxValue)
        {
            ArrayList h = new ArrayList();
            for (int i = 0; i <= maxValue; i++)
            {
                int number = 0;
                for (int j = 0; j < data.Count; j++)
                {
                   
                    if ((int)data[j] == i)
                    {
                        number++;  
                    }
                    
                }
                h.Add(number);
            }
            return h;
        }

        private double GetCenterHistogram(ArrayList histogram)
        {
            int sum = 0;
            for (int i = 0; i < histogram.Count; i++)
            {
                sum += i * (int)histogram[i];
            }
			if( histogram.Count > 1 )
				sum /=(histogram.Count-1);
            return sum;
        }

		private double GetLengthSignature(ArrayList xAxis,ArrayList yAxis)
		{
            double sum = 0;
			for(int i = 1; i < xAxis.Count; i++)
			{
                  sum += Math.Pow( (Math.Pow(((int)xAxis[i] - (int)xAxis[i-1]), 2) + Math.Pow(((int)yAxis[i] - (int)yAxis[i-1]), 2)),0.5);
			}

			return sum;
		}

		public static double SrednoAritmetichno(ArrayList data)
		{
			double result = 0;
			for(int i=0; i < data.Count; i++)
			{
				result += double.Parse(data[i].ToString());
			}

			if( data.Count != 0 )
				result /= data.Count;

			return result;
		}

		public static double SrednoKvadratichno(ArrayList data)
		{
          double result = 0;
			double srednoAritmetichno = SignCalculateData.SrednoAritmetichno(data);
			for( int i =0; i < data.Count; i++)
			{
				result += Math.Pow( (double.Parse(data[i].ToString())- srednoAritmetichno),2);
			}
			if ( data.Count >1 )
			{
				result /= (data.Count-1);
				result = Math.Pow( result,0.5);
			}

			return result;
		}

		



    }
}
